@@ -1,5 +1,5 @@
 /* This file is part of GNU Rush.                  
-   Copyright (C) 2008-2019 Sergey Poznyakoff
+   Copyright (C) 2008-2021 Sergey Poznyakoff
 
    GNU Rush is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -83,6 +83,29 @@ dump_null(char const *id, struct json_dumper *dmp)
 	fputs("null", dmp->fp);
 }
 
+void
+dumpstr(char const *string, FILE *fp)
+{
+	int c;
+
+	fputc('\"', fp);
+	for (; (c = *string) != 0; string++) {
+		int ec;
+		if (c == '\\' || c == '\"') {
+			fputc('\\', fp);
+			fputc(c, fp);
+		} else if (c_isprint(c))
+			fputc(c, fp);
+		else if ((ec = wordsplit_c_quote_char(c)) != 0) {
+			fputc('\\', fp);
+			fputc(ec, fp);
+		} else {
+			fprintf(fp, "\\%03o", c);
+		}
+	}
+	fputc('\"', fp);
+}
+
 static void
 dump_string_data(char const *string, struct json_dumper *dmp)
 {
@@ -90,13 +113,7 @@ dump_string_data(char const *string, struct json_dumper *dmp)
 		fputs("null", dmp->fp);
 		return;
 	}
-	fputc('\"', dmp->fp);
-	for (;*string; string++) {
-		if (*string == '\\' || *string == '\"')
-			fputc('\\', dmp->fp);
-		fputc(*string, dmp->fp);
-	}
-	fputc('\"', dmp->fp);
+	dumpstr(string, dmp->fp);
 }
 
 static void
@@ -311,7 +328,7 @@ dump_request(struct rush_request *req, FILE *fp)
 		else if (strcmp(ws.ws_wordv[i], "locale") == 0)
 			dump_string(req->i18n.locale, "locale", &dmp);
 		else if (strcmp(ws.ws_wordv[i], "environ") == 0)
-			dump_argv(environ, "environ", 1, &dmp);
+			dump_argv(req->env, "environ", 1, &dmp);
 		else if (strcmp(ws.ws_wordv[i], "vars") == 0)
 			dump_vars(req, "vars", &dmp);
 		else
