@@ -1,5 +1,5 @@
 /* This file is part of GNU Rush.                  
-   Copyright (C) 2008-2016 Sergey Poznyakoff
+   Copyright (C) 2008-2021 Sergey Poznyakoff
 
    GNU Rush is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -93,68 +93,118 @@ struct rush_map {
 	unsigned val_field;
 	char *defval;
 };
-	
+
+enum transform_target_type {
+	target_readonly,     /* Read-only variable */
+	target_command,      /* Command line */
+	target_program,      /* Executable program name */
+	target_arg,          /* Single command line argument */
+	target_var,          /* Variable */
+	target_env           /* Environment variable */
+};
+
+struct transform_target {
+	enum transform_target_type type;
+	union {
+		char *name;
+		struct {
+			int idx;
+			int ins;
+		} arg;
+	} v;
+};
+
 enum transform_node_type {
-	transform_cmdline,
-	transform_arg,
+	transform_set,
+	transform_delete,
 	transform_map,
-	transform_delarg,
-	transform_setcmd,
-	transform_setarg
+	transform_remopt,
+};
+
+struct option_defn {
+	char *s_opt;  /* short option name with optional argument designator */
+	char *l_opt;    /* optional long option name */
 };
 
 struct transform_node {
 	struct transform_node *next;
 	enum transform_node_type type;
-	int arg_no;
-	int progmod;
-	char *pattern;
+	struct transform_target target;
 	union {
-		transform_t trans;
-		struct rush_map map;
-		int arg_end;
+		struct {
+			char *pattern;
+			transform_t trans;
+		} xf;
+		struct rush_map map;    /* For transform_map */
+		int arg_end;            /* For tranform_delete, if target.type
+					   is target_arg */
+		struct option_defn remopt; /* For transform_remopt */
 	} v;
 };
+
+enum test_type {
+	test_cmpn,
+	test_cmps,
+	test_in,
+	test_group,
+	test_and,
+	test_or,
+	test_not,
+	test_fstest
+};
 
-/* Comparison operator */
 enum cmp_op {
 	cmp_eq,
 	cmp_ne,
 	cmp_lt,
 	cmp_le,
 	cmp_gt,
-	cmp_ge
+	cmp_ge,
+	cmp_match,
+	cmp_in
 };
 
-enum test_type {
-	test_cmdline,
-	test_arg,
-	test_argc,
-	test_uid,
-	test_gid,
-	test_user,
-	test_group,
+enum fstest_op {
+	fs_block_special = 'b',
+	fs_char_special  = 'c',
+	fs_directory     = 'd',
+	fs_exists        = 'e',
+	fs_regular       = 'f',
+	fs_set_gid       = 'g',
+	fs_owner_egid    = 'G',
+	fs_symlink       = 'h',
+	fs_sticky        = 'k',
+	fs_owner_euid    = 'O',
+	fs_pipe          = 'p',
+	fs_readable      = 'r',
+	fs_size          = 's',
+	fs_socket        = 'S',
+	fs_set_uid       = 'u',
+	fs_writable      = 'w',
+	fs_executable    = 'x'
 };
 
-struct test_numeric_node {
-	enum cmp_op op;
-	unsigned long val; /* FIXME: Should be uintmax_t? */
-};
-
-struct test_arg_node {
-	int arg_no;
-	regex_t regex;
-};
+typedef unsigned long rush_num_t;
 
 struct test_node {
-	struct test_node *next;
 	enum test_type type;
-	int negate;
 	union {
-		regex_t regex;
-		struct test_arg_node arg;
-		struct test_numeric_node num;
-		char **strv;
+		struct {
+			enum cmp_op op;
+			char *larg;
+			union {
+				char *str;
+				char **strv;
+				regex_t rx;
+				rush_num_t num;
+			} rarg;
+		} cmp;
+		char **groups;
+		struct test_node *arg[2];
+		struct {
+			int op;
+			char *arg;
+		} fstest;
 	} v;
 };
 
@@ -164,6 +214,7 @@ struct rush_sockaddr {
 };
 
 enum rush_three_state { rush_undefined = -1, rush_false, rush_true };
+typedef enum rush_three_state rush_bool_t;
 
 struct rush_i18n {
 	char *text_domain;           /* Gettext domain, if any */
@@ -171,8 +222,28 @@ struct rush_i18n {
 	char *locale;
 };	
 
+struct rush_error {     /* Rush error object */
+	int fd;         /* File descriptor to write to */
+	int idx;        /* Index of the standard error message, or -1 for
+			   user-defined message */
+};
+
+enum envar_type {
+	envar_set,   /* Set variable */
+	envar_unset, /* Unset variable(s) */
+	envar_keep,  /* Preserve variable(s) while clearing the environment */
+	envar_eval   /* Evaluate string for side effects. Discard the result */
+};
+
+struct envar {
+	struct envar *next;
+	enum envar_type type;
+	char *name;
+	char *value;
+};
+
 struct rush_rule {
-	struct rush_rule *next;      /* Next config in the list */
+	struct rush_rule *next;      /* Next rule in the list */
 
 	char *tag;
 	int fall_through;
@@ -183,17 +254,17 @@ struct rush_rule {
 	int line;                         /* and line number. */
 	
 	/* Match parameters */
-	struct test_node *test_head, *test_tail;
+	struct test_node *test_node;
 	
 	/* Transformation parameters: */
 	struct transform_node *transform_head, *transform_tail;
 
 	/* Environment modification: */
-	char **env;
+	int clrenv;
+	struct envar *envar_head, *envar_tail;
 
 	/* If not NULL, print this message on ERROR_FD and exit */
-	char *error_msg;
-	int error_fd;
+	struct rush_error *error;
 
 	struct rush_i18n i18n;
 	
@@ -209,6 +280,13 @@ struct rush_rule {
 	struct rush_sockaddr post_sockaddr;
 };
 
+struct rush_backref {
+	char *subject;
+ 	regmatch_t *match;
+	size_t nmatch;
+	size_t maxmatch;
+};
+
 struct rush_request {
         char *cmdline;         /* Command line */
         size_t argc;           /* Number of elements in argv */
@@ -225,6 +303,28 @@ struct rush_request {
 	enum rush_three_state acct;
 	const struct rush_sockaddr *post_sockaddr;
 	struct rush_i18n i18n;
+
+	/* Backreferences
+	   The backref field contains backreferences from the recent and
+	   penultimate regex matches. The backref_cur field indexes the
+	   recent backreferences. Remaining slot is used as a temporary
+	   storage during eventual next match. If that match succeeds, the
+	   value of backref_cur is updated to reflect the fact. The
+	   backref_count field keeps the actual number of backreferences
+	   used in the recent match. */
+	struct rush_backref backref[2]; 
+	int backref_cur;
+	size_t backref_count;
+
+	/* Constructed environment */
+	char **env;
+	size_t env_count;
+	size_t env_max;
+
+	/* Temporary variable storage */
+	char **var_kv;
+	size_t var_count;
+	size_t var_max;
 };
 
 #define PROGFILE(req) ((req)->prog ? (req)->prog : (req)->argv[0])
@@ -248,19 +348,27 @@ extern struct passwd *rush_pw;
 			logmsg(LOG_DEBUG, fmt, __VA_ARGS__);	\
 	} while(0)	
 
+struct cfloc;
+
 void die(enum error_type type, struct rush_i18n *i18n, const char *fmt, ...)
 	 RUSH_NORETURN RUSH_PRINTFLIKE(3,4);
+void die_usage(struct cfloc const *loc, char const *fmt, ...)
+	 RUSH_NORETURN RUSH_PRINTFLIKE(2,3);
 void logmsg(int prio, const char *fmt, ...)  RUSH_PRINTFLIKE(2,3);
+void vlogmsg(int prio, const char *fmt, va_list ap);
 
+enum {
+	lrec_ok = 0,
+	lrec_error = 1,
+	lrec_badval = 2
+};
+limits_record_t limits_record_create(void);
+int limits_record_add(limits_record_t lrec, char *str, char **endp);
 int parse_limits(limits_record_t *plrec, char *str, char **endp);
 int set_user_limits(const char *name, struct limits_rec *lrec);
 
-void parse_config(void);
-
-void set_error_msg(enum error_type type, char *text);
-int string_to_error_index(const char *name);
-
-transform_t compile_transform_expr (const char *expr, int cflags);
+transform_t compile_transform_expr (const char *expr, int cflags,
+				    struct cfloc *loc);
 char *transform_string (struct transform *tf, const char *input);
 
 int post_socket_send(const struct rush_sockaddr *sockaddr,
@@ -292,8 +400,21 @@ int cfck_keyword(const char *name);
 /* map.c */
 char *map_string(struct rush_map *map, struct rush_request *req);
 char *rush_expand_string(const char *string, struct rush_request *req);
+char **rush_getvarptr(struct rush_request *req, char const *varname);
+void rush_request_delvar(struct rush_request *req, char const *varname);
+enum transform_target_type rush_variable_target(char const *varname);
+void rush_ws_error (const char *fmt, ...);
 
 /* dump.c */
 void dump_request(struct rush_request *req, FILE *fp);
+
+/* rush_error management */
+void set_error_msg(enum error_type type, char *text);
+int string_to_error_index(const char *name);
+
+struct rush_error *new_standard_error(int fd, int idx);
+struct rush_error *new_error(int fd, char const *text, int unescape);
+char const *rush_error_msg(struct rush_error const *err,
+			   struct rush_i18n const *i18n);
 
 
